package com.persagy.apm.energyalarmstarter.alarmengine.jms;

import com.persagy.apm.energyalarmstarter.alarmengine.jms.model.DmpMessage;
import com.persagy.apm.energyalarmstarter.alarmengine.netty.NettyAlarmMsgBaseHandler;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @description:报警定义消息通知
 * @author:LuoGuangyi
 * @company:PersagyTechnologyCo.,Ltd
 * @since:2020/10/20 002016:30
 * @version:V1.0
 **/
@Slf4j
@Configuration
public class JmsConfig {
    /**
     * NettyAlarmMsgBaseHandler本身不进行ioc注入，这里注入的是他的子类。
     * 子类在实际项目中创建，starter中没有实例。
     */
    @Autowired
    private NettyAlarmMsgBaseHandler msgHandler;

    /**
     * 交换机
     */
    private final String exchange = "exchange-alarm-service";

    /**
     * 报警条件路由键
     */
    private final String alarmConditionRoutingKey = "alarm-condition-routing-key";

    /**
     * 报警对象与报警条件关联关系路由键
     */
    private final String objConditionRelRoutingKey = "obj-condition-rel-routing-key";

    /**
     * 报警条件队列
     */
    private final String conditionQueue = "condition-queue";

    /**
     * 设备与报警条件关联关系队列
     */
    private final String objConditionRelQueue = "obj-condition-rel-queue";

    @Bean
    public Queue conditionQueue() {
        return new Queue(conditionQueue, true);
    }

    @Bean
    public Queue objConditionRelQueue() {
        return new Queue(objConditionRelQueue, true);
    }

    @Bean
    public TopicExchange alarmServiceExchange() {
        return new TopicExchange(exchange);
    }

    @Bean
    public Binding conditionBinding() {
        return BindingBuilder.bind(conditionQueue()).to(alarmServiceExchange()).with(alarmConditionRoutingKey);
    }

    @Bean
    public Binding objConditionRelBinding() {
        return BindingBuilder.bind(objConditionRelQueue()).to(alarmServiceExchange()).with(objConditionRelRoutingKey);
    }

    @RabbitHandler
    @RabbitListener(queues = {conditionQueue, objConditionRelQueue})
    public void dealMsg(DmpMessage msg, Channel channel, Message message) throws Exception {
        try {
            log.debug("============================== Receive:" + msg);
            if (JmsTypeEnum.NEW_CONDITION.getType().equals(msg.getType())) {
                msgHandler.syncNewCondition(msg);
            }
            if (JmsTypeEnum.UPDATE_CONDITION.getType().equals(msg.getType())) {
                msgHandler.syncUpdatedCondition(msg);
            }
            if (JmsTypeEnum.DELETE_CONDITION.getType().equals(msg.getType())) {
                msgHandler.syncDeletedCondition(msg);
            }
            if (JmsTypeEnum.NEW_OBJ_CONDITION_REL.getType().equals(msg.getType())) {
                msgHandler.syncNewObjConditionRelList(msg);
            }
            if (JmsTypeEnum.DELETE_OBJ_CONDITION_REL.getType().equals(msg.getType())) {
                msgHandler.syncDeletedObjConditionRelList(msg);
            }
        } catch (Exception e) {
            log.error("消息消费失败，{}", e.getMessage());
        }
        channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
    }
}
